'''
Created on 22.06.2015

@author: lamour
'''
from algopy import zeros, cos, sin, exp

class RoboticArmStandard(object):
    """
    see S. L. Campbell and E. Griepentrog. Solvability of general differential algebraic equations. SIAM J. Sci. Comput., 16(2):257-270, 1995.
    """
    def __init__(self):
        self.type_of_DAE='standard'
# proper: f and d are given for f(d'(x,t),x,t)
# standard: f is given for f(x',x,t)
# linear: f is given as A(t)x'+B(t)x - q(t)
#         also self.eval_A and self.eval_B are available
        self.example_name='RoboticArm'
        #self.svdtol=1e-20

    def f(self,z,x,t):
        """ Robotic Arm, Index 5 DAE """
        f = zeros(8, dtype=x)
        #print 'in f ',type(t),type(x)
        
        def p1(u): 
            z=cos(exp(u)-1)+cos(u-1)
            return z
        
        def p2(u):
            z=sin(1-exp(u))+sin(1-u)
            return z
        
        def a(u):
            z=2/(2-cos(u)**2)
            return z
                   
        def b(u):
            z=cos(u)/(2-cos(u)**2)
            return z
             
        def c(u):
            z=sin(u)/(2-cos(u)**2)
            return z
             
        def d(u):
            z=cos(u)*sin(u)/(2-cos(u)**2)
            return z

        u1=x[6]
        u2=x[7]
       
        f[0]=z[0]-x[3]
        f[1]=z[1]-x[4]
        f[2]=z[2]-x[5]
        f[3]=z[3]-2*c(x[2])*(x[3]+x[5])**2-x[3]**2*d(x[2])-(2*x[2]-x[1])*(a(x[2])\
               +2*b(x[2]))-a(x[2])*u1+a(x[2])*u2
        f[4]=z[4]+2*c(x[2])*(x[3]+x[5])**2+x[3]**2*d(x[2])-(2*x[2]-x[1])*(1-3*a(x[2])\
               -2*b(x[2]))+a(x[2])*u1-(a(x[2])+1)*u2      
        f[5]=z[5]+2*c(x[2])*(x[3]+x[5])**2+x[3]**2*d(x[2])-(2*x[2]-x[1])*(a(x[2])\
                 -9*b(x[2]))+2*x[3]**2*c(x[2])+(x[3]+x[5])**2*d(x[2])+(a(x[2])+b(x[2]))*(u1-u2) 
        f[6]=cos(x[0])+cos(x[0]+x[2])-p1(t)
        f[7]=sin(x[0])+sin(x[0]+x[2])-p2(t)
        return f
    
    
    
if __name__=="__main__":        

    from numpy import array,exp as npexp, zeros as npzeros
    from DAEinitialization.consistentValues import consistentValuesIndex
    ex = RoboticArmStandard()
    n=8
  
    print 'Solution by minimization'

    t0=1.
    #x0=zeros((4,))
    expt0=npexp(t0)
    x1=1.-expt0
    x3=expt0-t0
    x4=-expt0
    x6=expt0-1.
    
    #alpha=array([x1, 0, x3,x4, 0, x6, 0, 0])
    alpha=array([x1, 0, x3,0, 0, 0, 0, 0])
    print 'alpha = ',alpha
    ti=t0
    ftola=1e-12
    K=6
    x0=npzeros(K*n)
    x0[:n]=alpha
    solution,index,cond,robust=consistentValuesIndex(ex,K,alpha,x0,ti,ftol=ftola,disp=True,info_level=2)

    
